package com.antai.sport.http.server.server.api.business.achievement.service;

import com.antai.sport.http.server.constants.AchievementCondition;
import com.antai.sport.http.server.constants.GameCurrencySourceType;
import com.antai.sport.http.server.constants.GameMode;
import com.antai.sport.http.server.repository.achievement.entity.Achievement;
import com.antai.sport.http.server.repository.achievement.entity.AchievementForSportUser;
import com.antai.sport.http.server.repository.achievement.mapper.AchievementForSportUserMapper;
import com.antai.sport.http.server.repository.sport.entity.SportUserSummary;
import com.antai.sport.http.server.server.api.business.achievement.dto.NewAchievementVO;
import com.antai.sport.http.server.server.api.business.achievement.mapper.AchievementBizMapper;
import com.antai.sport.http.server.server.api.business.sportusergamecurrency.service.SportUserGameCurrencyService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

@Service
public class AchievementService {

    @Resource
    private AchievementForSportUserMapper achievementForSportUserMapper;

    @Resource
    private AchievementBizMapper achievementBizMapper;

    @Resource
    private SportUserGameCurrencyService sportUserGameCurrencyService;

    /**
     * 处理签到类成就
     *
     * @param sportUserId
     */
    public void handleSignInAchievement(Long sportUserId, Integer continuousLoginDays) {
        this.initAchievement(sportUserId, AchievementCondition.SIGN_IN_DAYS, continuousLoginDays);
    }

    /**
     * 处理运动类成就
     * 先完成业务后面再考虑性能优化
     *
     * @param gameMode
     * @param summary
     */
    public void handleSportAchievement(Integer gameMode, SportUserSummary summary) {
        if (gameMode.equals(GameMode.GROUP_TRAINING)) {
            initAchievement(summary.getUserId(), AchievementCondition.TEAM_TRAINING,
                    summary.getTotalTeamTrainingFinishNum());
        }
        if (gameMode.equals(GameMode.SIMPLE_MATCH)) {
            initAchievement(summary.getUserId(), AchievementCondition.SIMPLE_MATCH,
                    summary.getTotalSimpleMatchFinishNum());
        }
        if (gameMode.equals(GameMode.ROOM_MODE)) {
            initAchievement(summary.getUserId(), AchievementCondition.ROOM_MODE, summary.getTotalGameModeFinishNum());
        }
        initAchievement(summary.getUserId(), AchievementCondition.DISTANCE, summary.getTotalDistance().intValue());
        initAchievement(summary.getUserId(), AchievementCondition.CLIMB_DISTANCE,
                summary.getTotalClimbDistance().intValue());
        initAchievement(summary.getUserId(), AchievementCondition.CALORIE, summary.getTotalCalorie());


    }

    /**
     * 成就计算和积分操作
     *
     * @param sportUserId
     * @param condition
     * @param conditionValue
     */
    private void initAchievement(Long sportUserId, Integer condition, Integer conditionValue) {
        if (conditionValue == null) {
            return;
        }
        List<Achievement> achievementList = achievementBizMapper.getAchievement(condition, sportUserId);
        for (Achievement achievement : achievementList) {
            if (conditionValue.compareTo(achievement.getConditionValue()) >= 0) {
                AchievementForSportUser achievementForSportUser = new AchievementForSportUser();
                achievementForSportUser.setSportUserId(sportUserId);
                achievementForSportUser.setAchievementId(achievement.getId());
                achievementForSportUser.setMarkRead(false);
                achievementForSportUser.setCreateTime(LocalDateTime.now());
                achievementForSportUserMapper.insert(achievementForSportUser);
                sportUserGameCurrencyService.addCurrency(sportUserId, achievement.getCurrency(),
                        GameCurrencySourceType.ACHIEVEMENT, achievementForSportUser.getId());
            }
        }
    }

    /**
     * 查询用户新成就
     *
     * @param sportUserId
     * @return
     */
    public List<NewAchievementVO> getNewAchievement(Long sportUserId) {
        List<NewAchievementVO> achievementVOList = achievementBizMapper.getNewAchievement(sportUserId);
        if (!achievementVOList.isEmpty()) {
            achievementBizMapper.updateReadAchievement(achievementVOList);
        }
        return achievementVOList;
    }

}
